<?php
class api4marketing {

    private $server;
    private $apiKey;
    private $sessionID;
    private $status;
    private $response;

    public function __construct($server,$apiKey) {

        $this->server       = $server;
        $this->apiKey       = $apiKey;

        $this->status       = false;
        $this->sessionID    = false;

        $this->login();
    }

    private function call($data = array(), $isJson = false) {

        if  ($isJson) {
            $argString = json_encode($data);
        } else {
            $argString = implode('&',$data);
        }
       // echo "<pre>";
        //var_dump($argString);
        $defaults = array(
            CURLOPT_POST => 1,
            CURLOPT_HEADER => 0,
            CURLOPT_URL => $this->server.'/api.php',
            CURLOPT_USERAGENT => '4marketing',
            CURLOPT_FRESH_CONNECT => 0,
            CURLOPT_RETURNTRANSFER => 1,
            CURLOPT_FORBID_REUSE => 0,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_SSL_VERIFYPEER => 0,
            CURLOPT_SSL_VERIFYHOST => 0,
            CURLOPT_POSTFIELDS => $argString
        );
        $ch = curl_init();

        curl_setopt_array($ch, $defaults);

        $response = curl_exec($ch);

        curl_close($ch);

        $response = json_decode($response);


        return $response;
    }

    public function login() {

        $login = array();
        $login[] = "Command=User.Login";
        $login[] = "ResponseFormat=JSON";
        $login[] = "Apikey=".$this->apiKey;

        $response = $this->call($login);

        $this->response = $response;

        if (!$response) {
            $this->status = false;
            return false;
        }
        if ($response->Success == true) {
            $this->status = true;
            $this->sessionID = $response->SessionID;
        }

        return $this->status;
    }

    public function getLists() {

        $lists = array();
        $lists[] = "Command=Lists.Get";
        $lists[] = "ResponseFormat=JSON";
        $lists[] = "SessionID=".$this->sessionID;
        $lists[] = "OrderField=ListID";
        $lists[] = "OrderType=ASC";

        $response = $this->call($lists);

        $this->response = $response;

        if ($response) {
            if ($response->Success == true) {
                return $response->Lists;
            }
        }
        return false;
    }
    public function udpList($listId){

        $ArrayPostParameters = array();
        $ArrayPostParameters[] = "SessionID=".$this->sessionID;
        $ArrayPostParameters[] = "Command=List.Update";
        $ArrayPostParameters[] = "ResponseFormat=JSON";
        $ArrayPostParameters[] = "SubscriberListID=".$listId;
        $ArrayPostParameters[] = "SyncInfo[active]=true";
        $ArrayPostParameters[] = "SyncInfo[domain]=".Context::getContext()->shop->domain;
        $ArrayPostParameters[] = "SyncInfo[shopid]=".Context::getContext()->shop->id;
        $ArrayPostParameters[] = "SyncInfo[module]=prestashop";
        $ArrayPostParameters[] = "SyncInfo[last_sync_date]=".(Configuration::get('4DEM_LAST_SYNC') ? strtotime(Configuration::get('4DEM_LAST_SYNC')) : 0);

        $response = $this->call($ArrayPostParameters);
        $this->response = $response;
        if ($response) {
            if ($response->Success == true) {
                return true;
            }
        }
        return $response->ErrorCode[0];
    }
    public function createList() {
        $params = array();
        $params[] = "Command=List.Create";
        $params[] = "ResponseFormat=JSON";
        $params[] = "SessionID=".$this->sessionID;
        $params[] = "SubscriberListName=".Context::getContext()->shop->domain." - ".Context::getContext()->shop->name." ID: ".Context::getContext()->shop->id;
        $params[] = "SyncInfo[active]=true";
        $params[] = "SyncInfo[domain]=".Context::getContext()->shop->domain;
        $params[] = "SyncInfo[shopid]=".Context::getContext()->shop->id;
        $params[] = "SyncInfo[module]=prestashop";
        $params[] = "SyncInfo[last_sync_date]=0";
        $response = $this->call($params);
        $this->response = $response;
        if ($response) {
            if ($response->Success == true) {
                Configuration::updateValue ( '4DEM_SELECTED_LIST_ID', $response->ListID );
                return true;
            }
        }
        return $response->ErrorCode[0];
    }

    public function createCustomFields($listId,$fields) {

        $err = array();
        /*CREAZIONE SCHEMA CAMPI CUSTOM*/
        $json = array();
        $err = array();
        foreach($fields as $key => $field){
            $customField = array();
            $customField[] = "Command=CustomField.Create";
            $customField[] = "ResponseFormat=JSON";
            $customField[] = "SessionID=".$this->sessionID;
            $customField[] = "SubscriberListID=".$listId;
            $customField[] = "FieldName=".$field["name"];
            $customField[] = "FieldType=".$field["type"];
            if($field["type"] == "Checkboxes"){
                foreach($field["options"] as $key2 => $opt){
                    $customField[] = "OptionLabel[$key2]=".$opt["label"];
                    $customField[] = "OptionValue[$key2]=".$opt["val"];

                }
            }
            $customField[] = "FieldDefautlValue=".$field["val"];
            $response = $this->call($customField);
            $json[$key]["ignore"] = false;
            $json[$key]["name"] = $field["name"];
            if($response->Success) {
                $json[$key]["id"] = $response->CustomFieldID;
            }else{
                $json[$key]["id"] =false;
                $err[] = "Errore per ".$key;
            }
        }
        $json["cellphone"]["ignore"] = false;
        $json["cellphone"]["name"] = "N° Cellulare";
        Configuration::updateValue ( '4DEM_CUSTOM_FIELDS', json_encode($json) );

        if (count($err)==0) {
            return true;
        }
        return $err;
    }
    public function updateCustomField($listId,$field,$fieldID) {
        /*CREAZIONE SCHEMA CAMPI CUSTOM*/
        $err = "";
        $customField = array();
        $customField[] = "Command=CustomField.Update";
        $customField[] = "ResponseFormat=JSON";
        $customField[] = "SessionID=".$this->sessionID;
        $customField[] = "SubscriberListID=".$listId;
        $customField[] = "FieldName=".$field["name"];
        $customField[] = "CustomFieldID=".$fieldID;
        $customField[] = "FieldType=".$field["type"];
        if($field["type"] == "Checkboxes"){
            foreach($field["options"] as $key2 => $opt){
                $customField[] = "OptionLabel[$key2]=".$opt["label"];
                $customField[] = "OptionValue[$key2]=".$opt["val"];

            }
        }
        $customField[] = "FieldDefautlValue=".$field["val"];
        $response = $this->call($customField);
        if(!$response->Success) {
            $err = "Errore per ".$key2;
        }

        if ($err=="") {
            return true;
        }
        return $err;
    }

    public function getCustomFields($listId) {

        $customFields = array();
        $customFields[] = "Command=CustomFields.Get";
        $customFields[] = "ResponseFormat=JSON";
        $customFields[] = "SessionID=".$this->sessionID;
        $customFields[] = "OrderField=FieldName";
        $customFields[] = "OrderType=ASC";
        $customFields[] = "SubscriberListID=".$listId;

        $response = $this->call($customFields);

        $this->response = $response;
        if ($response) {
            if ($response->Success == true) {
                return $response->CustomFields;
            }
        }
        return false;
    }

    public function subscribe($listId,$email,$mobile = "",$customFields = false, $customerInfo = false) {

        $subscribe = array();
        $subscribe[] = "Command=Subscriber.Subscribe";
        $subscribe[] = "ResponseFormat=JSON";
        $subscribe[] = "ListID=".$listId;
        $subscribe[] = "EmailAddress=".$email;
        if($mobile!="")
            $subscribe[] = "Mobile=".$mobile;

        if ($customFields) {
            foreach ($customFields as $key => $field) {
                $subscribe[] = "CustomField".$field."=".$customerInfo[$key];
            }
        }
        $subscribe[] = "IPAddress=".$_SERVER['REMOTE_ADDR'];

        $response = $this->call($subscribe);
        $this->response = $response;

//customump($response);die();
        if ($response) {
            if ($response->Success == true) {
                return true;
            }
        }
        return false;
    }

    public function getSubscriber($listId,$email) {
        $subscriberGet = array();
        $subscriberGet[] = "SessionID=".$this->sessionID;
        $subscriberGet[] = "Command=Subscribers.Get";
        $subscriberGet[] = "ResponseFormat=JSON";
        $subscriberGet[] = "OrderField=EmailAddress";
        $subscriberGet[] = "OrderType=DESC";
        $subscriberGet[] = "RecordsFrom=0";
        $subscriberGet[] = "RecordsPerRequest=1";
        $subscriberGet[] = "SearchField=EmailAddress";
        $subscriberGet[] = "SearchKeyword=".$email;
        $subscriberGet[] = "SubscriberListID=".$listId;
        $subscriberGet[] = "SubscriberSegment=Active";

        $response = $this->call($subscriberGet);

        $this->response = $response;

        if ($response) {
            if ($response->Success == true) {
                return $response->Subscribers[0];
            }
        }
        return false;
    }


    public function getSubscribersNumber($listId) {
        $subscriberGet = array();
        $subscriberGet[] = "SessionID=".$this->sessionID;
        $subscriberGet[] = "Command=Subscribers.Get";
        $subscriberGet[] = "ResponseFormat=JSON";
        $subscriberGet[] = "OrderField=EmailAddress";
        $subscriberGet[] = "OrderType=DESC";
        $subscriberGet[] = "RecordsFrom=0";
        $subscriberGet[] = "TotalSubscribers=1";
        //$subscriberGet[] = "SearchField=EmailAddress";
        //$subscriberGet[] = "SearchKeyword=".$email;
        $subscriberGet[] = "SubscriberListID=".$listId;
        $subscriberGet[] = "SubscriberSegment=Active";

        $response = $this->call($subscriberGet);
        $this->response = $response;

        if ($response) {
            if ($response->Success == true) {
                return $response->TotalSubscribers;
            }
        }
        return false;
    }


    public function updSubscriber($subscriberID,$listId,$email,$mobile = "", $customFields = false, $customerInfo = false) {

        $ArrayPostParameters = array();
        $ArrayPostParameters[] = "SessionID=".$this->sessionID;
        $ArrayPostParameters[] = "Command=Subscriber.Update";
        $ArrayPostParameters[] = "ResponseFormat=JSON";
        $ArrayPostParameters[] = "SubscriberID=".$subscriberID;
        $ArrayPostParameters[] = "SubscriberListID=".$listId;
        $ArrayPostParameters[] = "EmailAddress=".$email;
        $ArrayPostParameters[] = "Mobile=".$mobile;
        if ($customFields) {
            foreach ($customFields as $key => $field) {
                $ArrayPostParameters[] = "Fields[CustomField".$field."]=".$customerInfo[$key];
            }
        }

        $ArrayPostParameters[] = "Access=admin";

        $response = $this->call($ArrayPostParameters);

        $this->response = $response;

        if ($response) {
            if ($response->Success == true) {
                return true;
            }
        }
        return false;
    }

    public function unsubscribe($listId,$email) {

        $unsubscribe = array();
        $unsubscribe[] = "Command=Subscriber.Unsubscribe";
        $unsubscribe[] = "ResponseFormat=JSON";
        $unsubscribe[] = "SessionID=".$this->sessionID;
        $unsubscribe[] = "ListID=".$listId;
        $unsubscribe[] = "CampaignID=";
        $unsubscribe[] = "EmailAddress=".$email;
        $unsubscribe[] = "Preview=0";
        $unsubscribe[] = "IPAddress=".$_SERVER['REMOTE_ADDR'];

        $response = $this->call($unsubscribe);

        $this->response = $response;
//var_dump($response);die();
        if ($response) {
            if ($response->Success == true) {
                return true;
            }
        }
        return false;
    }
    public function getUnsubscribed($listId) {

        $subscriberGet = array();
        $subscriberGet[] = "SessionID=".$this->sessionID;
        $subscriberGet[] = "Command=Subscribers.Get";
        $subscriberGet[] = "ResponseFormat=JSON";
        $subscriberGet[] = "OrderField=UnsubscriptionDate";
        $subscriberGet[] = "OrderType=ASC";
        $subscriberGet[] = "RecordsFrom=0";
        $subscriberGet[] = "SubscriberListID=".$listId;
        $subscriberGet[] = "SubscriberSegment=Unsubscribed";

        $response = $this->call($subscriberGet);
        $this->response = $response;
//var_dump($response);die();
        if ($response) {
            if ($response->Success == true) {
                return $response->Subscribers;
            }
        }
        return false;
    }

    public function getStatus() {
        return $this->status;
    }

    public function getResponse()
    {
        return $this->response;
    }
}
?>