<?php

class blockfourdem_pshop extends Module {
	const SYNCH_ALL_CUSTOMERS = 0;
	const SYNCH_SPECIFIC_CUSTOMER = 1;
	/**
	 * ****************************************************************************
	 * Funzione __construct()
	 * Contiene tutte le informazioni del modulo visualizzate nel backend (nome, versione, descrizione).
	 *
	 * E' possibile selezionare il TAB all'interno del quale visualizzare il modulo
	 * *****************************************************************************
	 */
	function __construct() {
		$this->name = "blockfourdem_pshop";
		$this->tab = 'front_office_features'; // Products
		$this->version = '3.1.0';
		$this->need_instance = 0;
		$this->displayName = $this->l ( '4Dem for Prestashop' );
		$this->description = $this->l ( '4Dem Newsletter integration for Prestashop e-commerce.' );
		$this->author = "Tembo s.r.l.";
		$this->bootstrap = true;
		$this->ps_versions_compliancy = [
			'min' => '1.6.0'
		];
		parent::__construct ();
		if(Module::isInstalled("blockfourdem_pshop")){
			require_once "api4marketing_v2.php";
			if(Configuration::get ( '4DEM_CONSOLE_URL',"" )!="" &&  Configuration::get ( '4DEM_API_KEY',"" )!="")
				$this->api4marketingObject = new api4marketing ( Configuration::get ( '4DEM_CONSOLE_URL' ), Configuration::get ( '4DEM_API_KEY' ) );
			else
				$this->api4marketingObject = null;
		}

	}


	function install() {

		$sql = "
			CREATE TABLE `"._DB_PREFIX_."4dem_bridge` (
			  `id_user` int(11) unsigned DEFAULT NULL,
			  `id_4dem` int(11) unsigned DEFAULT NULL,
			  `email` varchar(128) DEFAULT '',
			  `error` text
			) ENGINE=InnoDB DEFAULT CHARSET=latin1;";
		$create = Db::getInstance ()->execute ( $sql );


		if (parent::install () == false
			|| 	!$create
			||  $this->registerHook ( 'actionObjectCustomerUpdateAfter' ) == false
			||  $this->registerHook ( 'actionObjectCustomerDeleteAfter' ) == false
			||  $this->registerHook ( 'actionObjectGroupUpdateAfter' ) == false
			||  $this->registerHook ( 'actionObjectGroupAddAfter' ) == false
			||  $this->registerHook ( 'actionObjectGroupDeleteAfter' ) == false)
			return false;
		Configuration::updateValue ( '4DEM_API_VERSION',2 );
		return true;
	}

	public function uninstall()
	{
		$sql ="DROP TABLE `"._DB_PREFIX_."4dem_bridge`";
		try {
			Db::getInstance ()->execute ( $sql );
		} catch (Exception $e) {
			$this->_errors[] = $e->getMessage();
		}
		Configuration::deleteByName('4DEM_API_VERSION');
		Configuration::deleteByName('4DEM_CONSOLE_URL');
		Configuration::deleteByName('4DEM_LAST_SYNC');
		Configuration::deleteByName('4DEM_API_KEY');
		Configuration::deleteByName('4DEM_CUSTOM_FIELDS');
		Configuration::deleteByName('4DEM_SELECTED_LIST_ID');
		Configuration::deleteByName('4DEM_WEBSERVER_UNSUBSCRIBE_ID');
		return parent::uninstall();
	}

	private function warning($string){
		if(Tools::version_compare(_PS_VERSION_,"1.6.1","<"))
			$output = $this->adminDisplayWarning($string);
		else
			$output = $this->displayWarning($string);
		return $output ;

	}

	public function getContent() {
		//$this->api4marketingObject->subscribe(Configuration::get("4DEM_SELECTED_LIST_ID"),"gabriele@tembo.it");
		if(Shop::isFeatureActive() && Context::getContext()->shop->getContext() != Shop::CONTEXT_SHOP){
			$this->warning($this->l('This module works only for a single shop. Select a shop to be able to configure it.'));
		}
		/**
		 * ****************************************************************************
		 * Funzione getContent()
		 * Prima funzione che viene caricata dal gestore dei moduli.
		 * Tools:getValue --> Legge il valore di un parametro in get o post
		 * Tools:isSubmit --> Verifica se il form è stato submittato (salvataggio parametri)
		 * A sua volta richiama la displayForm() che contiene le diettive dell'interfaccia di configurazione
		 *
		 * * Prestashop config variables
		 * 4DEM_API_KEY							-- Api Key console
		 * 4DEM_PASSWORD						-- Password console
		 * 4DEM_CONSOLE_URL						-- Url console
		 * 4DEM_CUSTOM_FIELDS					-- Mappatura Custom Fields per sapere quali ignorare
		 * 4DEM_MAIL_ERRORS						-- Eventuali fallimenti di import mail
		 * 4DEM_LAST_SYNC						-- Data ultima sincronizzazione
		 * *****************************************************************************
		 */

		// String variable per output errori
		$output = null;

		if (Tools::isSubmit ( 'submitUpdate' )) {
			// VARIABILE PER GESTIONE ERRORE INPUT


			// DEM_API_KEY VALUE
			$dem_api_key = strval ( Tools::getValue ( '4DEM_API_KEY' ) );
			if ($dem_api_key && !empty ( $dem_api_key ) && Validate::isGenericName ( $dem_api_key )){
				if(Configuration::get ( '4DEM_API_KEY' ) != $dem_api_key) {
					Configuration::deleteByName('4DEM_CUSTOM_FIELDS');

					Configuration::deleteByName('4DEM_SELECTED_LIST_ID');
				}
				Configuration::updateValue ( '4DEM_API_KEY', $dem_api_key );
			}

			// 4DEM 4DEM_CONSOLE_URL
			$dem_console_url = strval ( Tools::getValue ( '4DEM_CONSOLE_URL' ) );
			if ($dem_console_url && !empty ( $dem_console_url ) && Validate::isGenericName ( $dem_console_url )){
				if(Configuration::get ( '4DEM_CONSOLE_URL' ) != $dem_console_url) {
					Configuration::deleteByName('4DEM_CUSTOM_FIELDS');

					Configuration::deleteByName('4DEM_SELECTED_LIST_ID');
				}
				Configuration::updateValue ( '4DEM_CONSOLE_URL', $dem_console_url );
			}

			// 4DEM IGNORE FIELDS

			$fields = json_decode(Configuration::get ( '4DEM_CUSTOM_FIELDS'),true);
			if ($fields == null)
				$fields = array();
			foreach($fields as $key => $field){
				$val = strval ( Tools::getValue ( $key ) );
				$fields[$key]["ignore"] = false;
				if ($val){
					$fields[$key]["ignore"] = true;
				}
			}

			Configuration::updateValue ( '4DEM_CUSTOM_FIELDS', json_encode($fields) );
			$this->api4marketingObject = new api4marketing ( Configuration::get ( '4DEM_CONSOLE_URL' ), Configuration::get ( '4DEM_API_KEY' ) );

			$output .= $this->displayConfirmation ( $this->l ( 'Settings updated' ) );
		}

		// Nel caso in cui si sia scelto di riallineare tutto il repository, viene avviata la sincronizzazione
		if (Tools::isSubmit ( 'submitSync' )||Tools::isSubmit ( 'submitSyncAll' )) {
			$all = false;

			if(Tools::isSubmit ( 'submitSyncAll' ))
				$all = true;
			$this->SynchAllData ( self::SYNCH_ALL_CUSTOMERS , $all);
			$output .= $this->displayConfirmation ( $this->l ( 'Synchronization completed!' ) );
		}

		if (Tools::isSubmit ( 'createNewList' )) {
			$ret = $this->api4marketingObject->createList();
			if($ret !== true) {
				$err="";
				foreach($ret as $key => $msg){
					$err .= $key." - ".$msg[0]."<br>".$this->l('We suggest to delete (or rename) the list from the 4Dem Control Panel and create another one from this module.');
				}
				$output .= $this->displayError($err);
				return $output . $this->displayForm ();
			}
			$ret = $this->api4marketingObject->createCustomFields(Configuration::get ( '4DEM_SELECTED_LIST_ID' ),$this->createFieds());

			if($ret !== true) {
				$output .= $this->displayError($this->l('Custom fields creation Error with code: '.$ret));
				return $output . $this->displayForm ();
			}
			$ret = $this->api4marketingObject->createWebServiceUnsubscribe(Tools::getCurrentUrlProtocolPrefix().Tools::getHttpHost().$this->_path."/s2s/unsubscribe.php");
			if($ret!== false){
				Configuration::updateValue ( '4DEM_WEBSERVER_UNSUBSCRIBE_ID', $ret );
			}else{
				$output .= $this->displayError($this->l('An error occured during the creation of Unsubscription Webservice'));
				return $output . $this->displayForm ();
			}
			$output .= $this->displayConfirmation ( $this->l ( 'New list created' ) );
		}

		return $output . $this->displayForm ();
	}

	// CREA UN ARRAY DI CORRISPONDENZA PER I CAMPI PERSONALIZZATI
	public function createFieds(){
		$fields = array();

		// Campi Single Line

		$field["name"] = "Nome";
		$field["type"] = "Single line";
		$field["val"] = "NULL";
		$fields["name"] = $field;

		$field["name"] = "Cognome";
		$fields["surname"] = $field;

		$field["name"] = "Sesso";
		$fields["sex"] = $field;

		$field["name"] = "Lingua";
		$fields["language"] = $field;


		$field["name"] = "Guest";
		$fields["is_guest"] = $field;

		$field["name"] = "Paese";
		$fields["country"] = $field;

		$field["name"] = "Provincia";
		$fields["province"] = $field;

		$field["name"] = "Citta";
		$fields["city"] = $field;

		$field["name"] = "Cliente";
		$fields["client"] = $field;


		//Campi Data

		$field["name"] = "Data registrazione al sito";
		$field["type"] = "Date field";
		$fields["reg_date"] = $field;

		$field["name"] = "Data di nascita";
		$fields["birth_date"] = $field;

		$field["name"] = "Data iscrizione Newsletter";
		$fields["news_date"] = $field;

		$field["name"] = "Ultima connessione";
		$fields["last_conn"] = $field;

		$field["name"] = "Ultimo Ordine";
		$fields["last_order"] = $field;

		$field["name"] = "Gruppo";
		$field["type"] = "Checkboxes";
		$field["options"] = array();
		foreach(Group::getGroups(Context::getContext()->language->id) as $key => $gruppo){
			$opt = array();
			$opt["label"] = $gruppo["name"];
			$opt["val"] = $gruppo["id_group"];
			$field["options"][] = $opt;
		}
		$fields["group"] = $field;
		return $fields;
	}
	public function displayForm() {
		$connStatus="";
		$output="";
		$this->context->controller->addJS ( $this->_path . 'blockfourdem_pshop.js', 'all' );
		if (!isset($this->api4marketingObject) || $this->api4marketingObject==null || ! $this->api4marketingObject->getStatus ()) {
			$connStatus .= "Offline";
		} else {
			$connStatus .= "Online";
		}
		if(!isset($this->_html))
			$this->_html="";
		$this->_html .= '
		<style>
			fieldset input{
				max-width: 300px;
				display: inline-block !important;
			}
			.row{
				margin: 5px 0;

			}

			#error_warning {
				color: #ff0000;
				font-weight: bold;
				padding-top: 20px;
			}
			#import_errors {
				  background-color: #ffffff;
				display: inline-block;
				padding: 15px;
				display:none;
				width: 700px;
			}

			.error span {
				display: inline-block;
				width: 60%;
			}
			.error span.mail {
				font-weight: bold;
				width: 40%;
			}
			.error-head {
				font-size: 20px;
				font-weight: bold;
			}
			.error-head .mail {
				display: inline-block;
				width: 40%;
			}

			#alert_block_submit {
				margin:10px 0;
				display: none;
				background-color: #ff7979;
				border: 2px solid #ce1e1e;
				color: #ffffff;
				font-weight: bold;
				padding: 20px;
			}

			#confirm_submit {
				background-color: #ffffff;
				border: 2px solid;
				color: #000000;
				cursor: pointer;
				display: inline-block;
				font-weight: normal;
				margin: 0 10px;
				padding: 2px 15px !important;
			}
			#confirm_submit:hover {
				background-color: #999999;
				color: #ffffff;
			}

			#syncCount {
				display: inline-block;
				font-weight: bold;
				padding: 0 10px;
			}
			#syncCount span {
				color: #ff0000;
				font-size: inherit;
			}
			span.info {
				color: #aaaaaa;
				font-size: 10px;
			}
		</style>
		<script>
			$(document).ready(function(){
				$("#show_errors").click(function(){
					$.fancybox($("#import_errors"));
				});

			});
		</script>
		<div class="container">
		<form method="post" action="' . Tools::safeOutput ( $_SERVER ['REQUEST_URI'] ) . '" id="form_4dem">
			<fieldset>
				<legend><img src="' . $this->_path . 'logo.gif" />' . $this->l ( 'Settings' ) . '</legend>
				<h2>' . $this->l ( '4Dem Settings' ) . ' (' . $connStatus . ')</h2>
				<span class="info">'.$this->l('You can generate your API KEY in your 4Dem Panel clicking on the "General Options" button on the top-left side of the screen and selecting Api Key,').'</span>
				<br><br>
				<div class="row">
					<div class="col-md-2 col-sm-3 col-xs-4 text-right">
						<label style="margin-top:6px;">' . $this->l ( '4Dem Api Key' ) . '</label>
					</div>
					<div class="col-md-10 col-sm-9 col-xs-8">
						<input type="text" name="4DEM_API_KEY" value="' . Tools::getValue ( '4DEM_API_KEY', Configuration::get ( '4DEM_API_KEY' ) ) . '" />
					</div>

				</div>
				<div class="row">
					<div class="col-md-2 col-sm-3 col-xs-4 text-right">
						<label style="margin-top:6px;">' . $this->l ( '4Dem account url' ) . '</label>
					</div>
					<div class="col-md-10 col-sm-9 col-xs-8">
						<input type="text" name="4DEM_CONSOLE_URL" value="' . Tools::getValue ( '4DEM_CONSOLE_URL', Configuration::get ( '4DEM_CONSOLE_URL' ) ) . '" />
						&nbsp;&nbsp;<span class="info">(' . $this->l ( 'e.g. http://www.example.com' ) . ')</span>
					</div>

				</div>
				';

		// Test connection
		if (isset($this->api4marketingObject) && $this->api4marketingObject!=null && $this->api4marketingObject->getStatus ()) {
			// Get lists array from console

			$dem_console_lists = $this->api4marketingObject->getLists();

			if($dem_console_lists == false)
				$dem_console_lists = array();
			$psCompatibleList = false;
			foreach ( $dem_console_lists as $dem_console_listobject ) {
				$integration = $dem_console_listobject->integration;
				if($integration){
					if($integration->active==true && $integration->domain==Context::getContext()->shop->domain && $integration->shopid==Context::getContext()->shop->id && $integration->module=="prestashop" ) {
						$psCompatibleList=$dem_console_listobject->name;
						$customFields = $this->api4marketingObject->getCustomFields($dem_console_listobject->id);

						$fields = $this->createFieds();
						$json = array();
						if(count($customFields) >= count($fields)) {
							foreach($customFields as $customField){
								foreach($fields as $key => $field){
									if($customField->name == $field["name"]){
										$json[$key]["ignore"] = false;
										$json[$key]["id"] = $customField->id;
										$json[$key]["name"] = $field["name"];
										break;
									}
								}
							}
							if(count($json) != count($fields)){
								$output = $this->displayError($this->l('There is an error with the custom fields.'));
								$this->warning($this->l(' We suggest to delete the List from 4Dem Panel and recreate it with this module. Remember not to modify the Custom Fields form 4Dem Panel once the list is created.'));
							}
							if(!Configuration::get ( '4DEM_CUSTOM_FIELDS' ) || Configuration::get ( '4DEM_CUSTOM_FIELDS' ) == json_encode(array()) ) {
								$json["cellphone"]["ignore"] = false;
								$json["cellphone"]["name"] = "N° Cellulare";
								Configuration::updateValue('4DEM_CUSTOM_FIELDS', json_encode($json));
							}
						}else{
							$output = $this->displayError($this->l('There is an error with the custom fields.'));
							$this->warning($this->l(' We suggest to delete the List from 4Dem Panel and recreate it with this module. Remember not to modify the Custom Fields form 4Dem Panel once the list is created.'));

						}

						if(!Configuration::get ( '4DEM_SELECTED_LIST_ID')) {
							Configuration::updateValue('4DEM_SELECTED_LIST_ID', $dem_console_listobject->id);
						}
						if(isset($integration->last_sync_date)){
							Configuration::updateValue('4DEM_LAST_SYNC', date("Y-m-d H:i:s",$integration->last_sync_date));
						}
					}
				}

			}
			if(!$psCompatibleList) {
				//Configuration::deleteByName ( '4DEM_SELECTED_LIST_ID');
				$this->_html .= '
					<div class="clear"></div>
					<h2>' . $this->l ( 'Customer subscribers Setting' ) . '</h2>
					<div class="row">
						<div class="col-md-2 col-sm-3 col-xs-4 text-right">
							<label>' . $this->l('Create a new Prestashop compatible list') . '</label>
						</div>
						<div class="col-md-10 col-sm-9 col-xs-8">
							<input type="submit" name="createNewList" value="' . $this->l('Create') . '" />
						</div>
					</div>
					';
			}else{
				$this->_html .= '
					<div class="row">
						<div class="col-md-2 col-sm-3 col-xs-4 text-right">
							<label>' . $this->l('List for this shop') . '</label>
						</div>
						<div class="col-md-10 col-sm-9 col-xs-8">
							<p><strong style="color: black; font-size: 15px;">'.$psCompatibleList.'</strong></p>
						</div>
					</div>
					';

			}
			if( $psCompatibleList && Configuration::get ( '4DEM_SELECTED_LIST_ID' ) &&  Configuration::get ( '4DEM_SELECTED_LIST_ID' ) != 0) {
				$fields = json_decode(Configuration::get ( '4DEM_CUSTOM_FIELDS' ),true);
				$this->_html .= '
					<h2>' . $this->l('Ignore rules (check = ignore)') . '</h2>
					<span class="info">'.$this->l('Check the fields you want to ignore, they will not be uploaded on 4Dem platform.').'</span>
					';
				if($fields != null) {

					foreach ($fields as $key => $field) {
						$this->_html .= '
					<div class="row">
						<div class="col-md-2 col-sm-3 col-xs-4 text-right">
							<label for ="check_' . $key . '">' . $field["name"] . ' </label>
						</div>
						<div class="col-md-10 col-sm-9 col-xs-8">
							<input type="checkbox" id="check_' . $key . '" name="' . $key . '" value="1" ' . ($field["ignore"] ? 'checked="checked" ' : '') . '/>
						</div>
					</div>';
					}
				}else {
					$this->_html .= '
					<div class="row">
						<div class="col-md-2 col-sm-3 col-xs-4 text-right">
							<label>' . $this->l('Error in custom fields') . '</label>
						</div>
						<div class="col-md-10 col-sm-9 col-xs-8">
							' . $this->l('We suggest to delete the list from 4Dem panel and recreate it through this prestashop Module') . '
						</div>
					</div>';
				}
				$this->_html.='
					<h2>' . $this->l ( 'Cronjob settings' ) . '</h2>
					<span class="info">'.$this->l('In order to create an automatic synchronizazion we suggest you to set a cronjob that calls, once a day during the night, this URL').'</span><br><br>
					<div class="row">
						<div class="col-md-2 col-sm-3 col-xs-4 text-right">
							<label>' . $this->l ( 'Cron URL' ) . ':</label>
						</div>
						<div class="col-md-10 col-sm-9 col-xs-8">
							wget -q --spider -t 3600 '.Tools::getCurrentUrlProtocolPrefix().Context::getContext()->shop->domain.Context::getContext()->shop->physical_uri.'modules/blockfourdem_pshop/cron/cron.php?secure_key='.md5(_COOKIE_KEY_.Configuration::get('PS_SHOP_NAME')).'
						</div>
					</div>';
			}

		}
		$this->_html .= '
<br>
<br>
			<div id="alert_block_submit">
				' . $this->l('ATTENTION!') . '<br>
				' . $this->l('The script may takes several minutes to complete, you can press Esc while the page is loading to keep operating on your website, the script will be executed anyway. Click OK to proceed ->') . '
				 <p id="confirm_submit">OK</p>
				</div>
					<div class="clear"></div>
			<div class=" clear pspace">
				<input type="submit" name="submitUpdate" id="submitUpdate"' . 'value="' . $this->l('Save and Update') . '" class="button" />&nbsp;&nbsp;';
		$this->_html .= '<input type="submit" name="onlyUpdate" id="onlyUpdate"' . 'value="' . $this->l('Update') . '" class="button" />';
		if ($connStatus == "Online" && Configuration::get ( '4DEM_SELECTED_LIST_ID' ) && Configuration::get ( '4DEM_SELECTED_LIST_ID' )!="") {
			if(Configuration::get('4DEM_LAST_SYNC') && Configuration::get('4DEM_LAST_SYNC') != 0)
				$this->_html .= '<input type="submit" name="submitSync" value="' . $this->l('Update Synchronization') . '" class=" button" style="margin-left: 20px;"/>';
			$this->_html .= '<input type="submit" name="submitSyncAll" value="' . $this->l('Syncronize All') . '" class="button_sync button" />';
			$actual = $this->api4marketingObject->getSubscribersNumber(Configuration::get ( '4DEM_SELECTED_LIST_ID' ));
			if($actual == "")
				$actual = 0;
			$total = $this->getSubscribedCustomers();
			$this->_html .= '<p id="syncCount">' . $this->l('Customers in list') ." <span style='color:#555;'>". $actual . '</span>  - ' . $this->l('Last synchronization') . ': ' . ( Configuration::get('4DEM_LAST_SYNC') ?  date("d M Y H:i:s",strtotime(Configuration::get('4DEM_LAST_SYNC')))  : 'Never' ).'</p>';


		}
		'</div>
			</fieldset>
		</form>';

		$sql="SELECT c.email,b.error FROM "._DB_PREFIX_."4dem_bridge as b JOIN "._DB_PREFIX_."customer as c ON b.id_user = c.id_customer WHERE error <> null OR error <> ''";
		$ret=Db::getInstance ()->executeS( $sql );
		if($ret){
			$string = "<p id='error_warning'>Si sono verificati ".count($ret)." errori durante l'importazione dei contatti: <a id='show_errors'>Mostra errori di importazione</a></p><div id='import_errors'> ";
			$string.='
				<p class="error-head">
					<span class="mail">E-Mail</span><span>Error</span>
				</p>
				';
			foreach ( $ret as $key => $row ) {
				$string.='
				<p class="error">
					<span class="mail">'.$row["email"].'</span><span>'.$row["error"].'</span>
				</p>
				';
			}
			$string .= "</div> ";
			$this->_html .= $string;
		}
		$this->_html .= '</div>';

		return $output . $this->_html;
	}
	public function getSubscribedCustomers(){

		$sql = 'SELECT COUNT(*)
				FROM `'._DB_PREFIX_.'customer`
				WHERE active = 1 AND deleted = 0 AND newsletter = 1
				ORDER BY `email` ';
		return Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);
	}

	public function hookActionObjectCustomerDeleteAfter($params) {
		if($params["object"]->newsletter == 1) {
			$api4marketingObject = new api4marketing (Configuration::get('4DEM_CONSOLE_URL'), Configuration::get('4DEM_API_KEY'));
			$api4marketingSubs = $api4marketingObject->getSubscriber(Configuration::get('4DEM_SELECTED_LIST_ID'), $params["object"]->email);
			if ($api4marketingSubs) {
				$api4marketingObject->unsubscribe(Configuration::get('4DEM_SELECTED_LIST_ID'), $params["object"]->email);
			}
		}
	}
	public function hookActionObjectGroupDeleteAfter($params) {
		$customFields = json_decode(Configuration::get('4DEM_CUSTOM_FIELDS'));
		if(count($customFields)!=0){
			$groupCustom = $customFields->group->id;
			$api4marketingObject = new api4marketing (Configuration::get('4DEM_CONSOLE_URL'), Configuration::get('4DEM_API_KEY'));
			$fields = $api4marketingObject->getCustomFields(Configuration::get("4DEM_SELECTED_LIST_ID"));
			$field["name"] = "Gruppo";
			$field["type"] = "Checkboxes";
			$field["options"] = array();
			foreach(Group::getGroups(Context::getContext()->language->id) as $key => $gruppo){
				$opt = array();
				$opt["label"] = $gruppo["name"];
				$opt["val"] = $gruppo["id_group"];
				$field["options"][] = $opt;
			}
			$api4marketingObject->updateCustomField(Configuration::get("4DEM_SELECTED_LIST_ID"),$field,$groupCustom);
		}
	}
	public function hookActionObjectGroupUpdateAfter($params) {
		$customFields = json_decode(Configuration::get('4DEM_CUSTOM_FIELDS'));
		if(count($customFields)!=0){
			$groupCustom = $customFields->group->id;
			$api4marketingObject = new api4marketing (Configuration::get('4DEM_CONSOLE_URL'), Configuration::get('4DEM_API_KEY'));
			$fields = $api4marketingObject->getCustomFields(Configuration::get("4DEM_SELECTED_LIST_ID"));
			$field["name"] = "Gruppo";
			$field["type"] = "Checkboxes";
			$field["options"] = array();
			foreach(Group::getGroups(Context::getContext()->language->id) as $key => $gruppo){
				$opt = array();
				$opt["label"] = $gruppo["name"];
				$opt["val"] = $gruppo["id_group"];
				$field["options"][] = $opt;
			}
			$api4marketingObject->updateCustomField(Configuration::get("4DEM_SELECTED_LIST_ID"),$field,$groupCustom);
		}
	}
	public function hookActionObjectGroupAddAfter($params) {
		$customFields = json_decode(Configuration::get('4DEM_CUSTOM_FIELDS'));
		if(count($customFields)!=0){
			$groupCustom = $customFields->group->id;
			$api4marketingObject = new api4marketing (Configuration::get('4DEM_CONSOLE_URL'), Configuration::get('4DEM_API_KEY'));
			$fields = $api4marketingObject->getCustomFields(Configuration::get("4DEM_SELECTED_LIST_ID"));
			$field["name"] = "Gruppo";
			$field["type"] = "Checkboxes";
			$field["options"] = array();
			foreach(Group::getGroups(Context::getContext()->language->id) as $key => $gruppo){
				$opt = array();
				$opt["label"] = $gruppo["name"];
				$opt["val"] = $gruppo["id_group"];
				$field["options"][] = $opt;
			}
			$api4marketingObject->updateCustomField(Configuration::get("4DEM_SELECTED_LIST_ID"),$field,$groupCustom);
		}

	}
	public function hookActionObjectCustomerUpdateAfter($params) {
		$sql = "SELECT id_4dem FROM "._DB_PREFIX_."4dem_bridge WHERE email = '".$params["object"]->email."'";
		$id = Db::getInstance ()->getValue( $sql );
		if($params["object"]->newsletter == 0) {
			$api4marketingSubs = $this->api4marketingObject->getSubscriber(Configuration::get('4DEM_SELECTED_LIST_ID'), $id);
			if ($api4marketingSubs) {
				$this->api4marketingObject->unsubscribe(Configuration::get('4DEM_SELECTED_LIST_ID'), $id);
			}
		}

	}
	public function CronSync(){
		$this->SynchAllData ( self::SYNCH_ALL_CUSTOMERS );
	}
	private function SynchAllData($synchMode, $all = false) {
		$dateSync = Configuration::get('4DEM_LAST_SYNC');
		if($all)
			$dateSync = 0;

		/**
		 * ****************************************************************************
		 * Funzione synch($allData = false, $singleCustomerID = -1)
		 * Sincronizza tutti i dati di Prestashop con la console
		 *
		 * *****************************************************************************
		 */
		/*
		 * SELECT CHE INDIVDUA 1. Tutti gli utenti della tabella customers che hanno fatto almeno un ordine 2. La data dell'ultimo ordine effettuato 3. Tutti gli utenti che, seppur registrati, non hanno mai fatto un ordine
		 */
		// Se Fallisce l'inizializzazione, esce di brutto
		$api4marketingObject = new api4marketing ( Configuration::get ( '4DEM_CONSOLE_URL' ), Configuration::get ( '4DEM_API_KEY' ) );
		if (! $api4marketingObject->getStatus ()) {

			return false;
		}
		/*$unsubscribed = $api4marketingObject->getUnsubscribed (Configuration::get('4DEM_SELECTED_LIST_ID'));
		if($unsubscribed) {
			foreach ($unsubscribed as $user) {

				if (strtotime($user->UnsubscriptionDate) > strtotime($dateSync) && Customer::customerExists($user->EmailAddress)) {
					$customer = new Customer(Customer::customerExists($user->EmailAddress, true));
					if ($customer) {
						$customer->newsletter = 0;
						$customer->update();
					}
				} else {
					break;
				}
			}
		}
*/
		if (Configuration::get ( '4DEM_SELECTED_LIST_ID' ) && (($synchMode == self::SYNCH_ALL_CUSTOMERS) || ($synchMode == self::SYNCH_SPECIFIC_CUSTOMER && $singleCustomerID > - 1))) {

			$sql =
				(($dateSync != NULL) ? "SELECT * FROM (" : "") . "
				SELECT ".
				(($dateSync != NULL) ? " customersData.date_upd as upd_cust, " . _DB_PREFIX_ . "address.date_upd as upd_address," : "") . "
 				customersData.id_customer,
 				GROUP_CONCAT(DISTINCT " . _DB_PREFIX_ . "customer_group.id_group) as gruppi,
 				" . _DB_PREFIX_ . "lang.iso_code AS lingua,
 				" . _DB_PREFIX_ . "state.iso_code AS provincia,
 				" . _DB_PREFIX_ . "address.city,
 				" . _DB_PREFIX_ . "country_lang.name AS paese,
 				customersData.is_guest, IF (customersData.id_gender = 1 , \"M\" , IF (customersData.id_gender = 2 , \"F\" , \"NS\")) AS gender ,
 				customersData.firstname,
 				customersData.lastname,
 				customersData.email,
 				'".((isset($_SERVER['REMOTE_ADDR'])) ? $_SERVER['REMOTE_ADDR'] : '0.0.0.0')."' ip_registration_newsletter,
 				( SELECT c.date_add FROM " . _DB_PREFIX_ . "guest g LEFT JOIN " . _DB_PREFIX_ . "connections c ON c.id_guest = g.id_guest WHERE g.id_customer = customersData.id_customer ORDER BY c.date_add DESC LIMIT 1 ) as ultima_connessione ,
 				customersData.newsletter_date_add,
 				" . _DB_PREFIX_ . "address.phone_mobile,
 				customersData.birthday,
 				customersData.date_add AS registrazione,
 				customersOrders.lastOrderPlaced
 			FROM " . _DB_PREFIX_ . "customer customersData
 				LEFT JOIN " . _DB_PREFIX_ . "lang ON customersData.id_lang = " . _DB_PREFIX_ . "lang.id_lang
 				LEFT JOIN " . _DB_PREFIX_ . "address ON customersData.id_customer = " . _DB_PREFIX_ . "address.id_customer
 				LEFT JOIN " . _DB_PREFIX_ . "country_lang ON " . _DB_PREFIX_ . "address.id_country = " . _DB_PREFIX_ . "country_lang.id_country AND customersData.id_lang = " . _DB_PREFIX_ . "country_lang.id_lang
 				LEFT JOIN " . _DB_PREFIX_ . "state ON " . _DB_PREFIX_ . "address.id_state = " . _DB_PREFIX_ . "state.id_state
 				LEFT JOIN " . _DB_PREFIX_ . "customer_group ON " . _DB_PREFIX_ . "customer_group.id_customer = customersData.id_customer
 				LEFT JOIN (
 				SELECT id_customer, date(MAX(date_add)) AS lastOrderPlaced
 					FROM " . _DB_PREFIX_ . "orders
 				GROUP BY id_customer ) customersOrders ON customersData.id_customer = customersOrders.id_customer

 			WHERE customersData.newsletter = 1
 					AND customersData.active = 1
 					AND customersData.deleted = 0
 					". (($synchMode == self::SYNCH_ALL_CUSTOMERS || $singleCustomerID == - 1) ? "" : "AND customersData.id_customer = $singleCustomerID ") . "
 			GROUP BY customersData.id_customer
 			". (($dateSync != NULL) ? ') AS tab  WHERE upd_cust > "'.$dateSync.'" OR ultima_connessione > "'.$dateSync.'" OR lastOrderPlaced > "'.$dateSync.'" OR upd_address > "'.$dateSync.'"' : "") . "
 			ORDER BY email ";
			/**
			 * **********************************************************************************
			 * ESECUZIONE DEL CODICE CHE
			 * 1 TENTA DI EFFETTUARE L'INSERIMENTO
			 * 2 SE NON VA A BUON FINE (ESISTE GIA') ESEGUE UN UPDATE
			 * **********************************************************************************
			 */
			// echo $sql . "<br/><br/>";
			$result_data = Db::getInstance ()->executeS ( $sql );
			$sql = 'SELECT  email,
							ip_registration_newsletter,
							null firstname ,
							null lastname , 
							null gender , 
							null lingua ,
							null is_guest , 
							null paese , 
							null provincia , 
							null city , 
							null registrazione , 
							null birthday , 
							newsletter_date_add , 
							null ultima_connessione ,    
							null lastOrderPlaced ,    
							null gruppi ,    
							concat("n-",id) id_customer, 
							null phone_mobile     
						FROM '._DB_PREFIX_.'newsletter 
						WHERE active = 1
						AND email NOT IN (
							SELECT email
							FROM ' . _DB_PREFIX_ . 'customer
							WHERE deleted != 1
						)
						AND id_shop = '.Context::getContext()->shop->id . '
						AND id_shop_group = '.Context::getContext()->shop->id_shop_group.';';
			$result_data2 = Db::getInstance ()->executeS ( $sql );
			$result_data = array_merge($result_data,$result_data2);
			if ($result_data) {
				$operations = array();
				$counter = 0;
				foreach ( $result_data as $key => $row ) {
					// Use custom fields data
					$subscribeFields = array ();
					$subscribeValues = array ();
					$fields = json_decode(Configuration::get ( "4DEM_CUSTOM_FIELDS" ),true);


					if (!$fields["name"]["ignore"]) {
						$subscribeFields [] = $fields["name"]["id"];
						$subscribeValues [] = ($row ['firstname'] != null ? $row ['firstname'] : "-");
					}
					if (!$fields["surname"]["ignore"]) {
						$subscribeFields [] = $fields["surname"]["id"];
						$subscribeValues [] = ($row ['lastname'] != null ? $row ['lastname'] : "-");
					}
					if (!$fields["sex"]["ignore"]) {
						$subscribeFields [] = $fields["sex"]["id"];
						$subscribeValues [] = ($row ['gender'] != null ? $row ['gender'] : "-");
					}
					if (!$fields["language"]["ignore"]) {
						$subscribeFields [] = $fields["language"]["id"];
						$subscribeValues [] = ($row ['lingua'] != null ? $row ['lingua'] : "-");
					}
					if (!$fields["is_guest"]["ignore"]) {
						$subscribeFields [] = $fields["is_guest"]["id"];
						$subscribeValues [] = ($row ['is_guest'] != null ? $row ['is_guest'] : "-");
					}
					if (!$fields["country"]["ignore"]) {
						$subscribeFields [] = $fields["country"]["id"];
						$subscribeValues [] = ($row ['paese'] != null ? $row ['paese'] : "-");
					}
					if (!$fields["province"]["ignore"]) {
						$subscribeFields [] = $fields["province"]["id"];
						$subscribeValues [] = ($row ['provincia'] != null ? $row ['provincia'] : "-");
					}
					if (!$fields["city"]["ignore"]) {
						$subscribeFields [] = $fields["city"]["id"];
						$subscribeValues [] = ($row ['city'] != null ? $row ['city'] : "-");
					}
					if (!$fields["client"]["ignore"]) {
						$subscribeFields [] = $fields["client"]["id"];
						$subscribeValues [] = ($row ['id_customer'] == null ? "NO" : "YES");
					}
					if (!$fields["cellphone"]["ignore"]) {
						$mobile = $row ['phone_mobile'];
						if(strlen($mobile)==10)
							$mobile="+39".$mobile;
						else if(strlen($mobile) == 11){
							$mobile="+".$mobile;
						}
						if(strlen($mobile) != 13){
							$mobile="";
						}
						$mobile = urlencode($mobile);
					}
					if (!$fields["reg_date"]["ignore"]) {
						$subscribeFields [] = $fields["reg_date"]["id"];
						$subscribeValues [] = ($row ['registrazione'] != null ? $row ['registrazione'] : "-");
					}
					if (!$fields["birth_date"]["ignore"]) {
						$subscribeFields [] = $fields["birth_date"]["id"];
						$subscribeValues [] = ($row ['birthday'] != null ? $row ['birthday'] : "-");
					}
					if (!$fields["news_date"]["ignore"]) {
						$subscribeFields [] = $fields["news_date"]["id"];
						$subscribeValues [] = ($row ['newsletter_date_add'] != null ? $row ['newsletter_date_add'] : "-");
					}
					if (!$fields["last_conn"]["ignore"]) {
						$subscribeFields [] = $fields["last_conn"]["id"];
						$subscribeValues [] = ($row ['ultima_connessione'] != null ? $row ['ultima_connessione'] : "-");
					}
					if (!$fields["last_order"]["ignore"]) {
						$subscribeFields [] = $fields["last_order"]["id"];
						$subscribeValues [] = ($row ['lastOrderPlaced'] != null ? $row ['lastOrderPlaced'] : "-");
					}

					if (!$fields["group"]["ignore"]) {
						$gruppi = explode(",",$row["gruppi"]);
						$subscribeFields [] = $fields["group"]["id"];
						$val = "";
						foreach($gruppi as $gruppo){
							$val.=$gruppo."||||";
						}
						$subscribeValues [] = rtrim($val,"||||");

					}
//ddd($api4marketingObject->subscribe ( Configuration::get ( '4DEM_SELECTED_LIST_ID' ), $row ['email'],$mobile, $subscribeFields, $subscribeValues ));
					// print_r($subscribeValues);
					// Verifica se esiste l'indirizzo email. Se esiste fa un update, altrimenti una insert
					/*$api4marketingSubs = $api4marketingObject->getSubscriber ( Configuration::get ( '4DEM_SELECTED_LIST_ID' ), $row ['email'] );
					if (! $api4marketingSubs) {
						if(!$api4marketingObject->subscribe ( Configuration::get ( '4DEM_SELECTED_LIST_ID' ), $row ['email'],$mobile, $subscribeFields, $subscribeValues )){
							Configuration::updateValue( '4DEM_MAIL_ERRORS',Configuration::get( '4DEM_MAIL_ERRORS').",".$row ['email']);
						}
					} else {
						$api4marketingObject->updSubscriber ( $api4marketingSubs->SubscriberID, Configuration::get ( '4DEM_SELECTED_LIST_ID' ), $row ['email'], $mobile, $subscribeFields, $subscribeValues );
					}*/

					$command["command"] = "Subscriber.Store";
					if($row["id_customer"])
						$command["custom_id"] = $row["id_customer"] ;
					$ip = '0.0.0.0';
					if($row["ip_registration_newsletter"]!= null && $row["ip_registration_newsletter"] != "")
						$ip = $row["ip_registration_newsletter"];
					$command["body"] = $this->generateCustomerData($row ['email'],$mobile, $subscribeFields, $subscribeValues,$ip );
					$operations[] = $command;
					if($counter == 1000){
						$counter= 0;
						$id_batch = $api4marketingObject->massiveImport(Tools::getCurrentUrlProtocolPrefix().Tools::getHttpHost().$this->_path."/s2s/callback.php",$operations);
						Configuration::updateValue ( '4DEM_LAST_BATCH', "id: ".$id_batch." - sync: ".$key." - ".date("d-m-Y h:i:s") );
						$operations = array();
						sleep(1);
					}else{
						$counter++;
					}
				}
				if($counter < 1000) {
					$id_batch= $api4marketingObject->massiveImport(Tools::getCurrentUrlProtocolPrefix().Tools::getHttpHost().$this->_path."/s2s/callback.php", $operations);
					Configuration::updateValue('4DEM_LAST_BATCH', "id: " . $id_batch . " - sync: " . $key . " - " . date("d-m-Y h:i:s"));

				}
				Configuration::updateValue( '4DEM_LAST_SYNC',date("Y-m-d H:i:s"));
				$api4marketingObject->updList(Configuration::get ( '4DEM_SELECTED_LIST_ID' ));
			}
		}
	}

	private function generateCustomerData($email,$mobile = "",$customFields = false, $customerInfo = false,$ip = false) {


		$subscribe = array();
		$subscribe["update_if_duplicate"] = true;
		$subscribe["email_address"] = $email;
		if($mobile!="" & $mobile!= null)
			$subscribe["mobile"] = urldecode($mobile);

		if ($customFields) {
			$subscribe["custom_fields"] = array();
			foreach ($customFields as $key => $field) {
				$subscribe["custom_fields"][$key]["id"] = $field;
				$subscribe["custom_fields"][$key]["value"] = $customerInfo[$key];
			}
		}
		$subscribe["subscription"]["ip"] = $ip ? $ip : Tools::getRemoteAddr();
		$subscribe["recipient_id"] = Configuration::get ( '4DEM_SELECTED_LIST_ID' );

		return $subscribe;
	}

}
?>
